#!/usr/bin/perl -w
# extract_web.pl
# Emmanuel Jeannot, INRIA: emmanuel.jeannot@inria.fr
# Browse the http://genealogy.math.ndsu.nodak.edu site and output the genealogy DAG o the corresponding ID

use LWP;

#Hastable to check if we already browse a given ID
%id_hash=();


#Transform some HTML code into text one.
sub html2txt{
   $res=$_[0];
   $res =~ s/&quot;/"/g;
   $res =~ s/&nbsp;//g;
   $res =~ s/&nbsp//g;
   $res =~ s/&Eacute;/E/g;
   $res =~ s/&eacute;//g;
   $res =~ s/&agrave;//g;
   $res =~ s/&egrave;//g;
   $res =~ s/&Egrave;/E/g;
   $res =~ s/&auml;//g;
   $res =~ s/&uuml;//g;
   $res =~ s/&amp;/--/g;
   $res =~ s//\\"{u}/g;
   $res =~ s//{\\~a}/g;
   $res =~ s//{\\~n}/g;
   $res =~ s/&lt;/</g;
   $res =~ s/&ouml;//g;
   $res =~ s/\\"{u}//g;
   $res =~ s/&Uuml;//g;
   $res =~ s/&szlig;/sz/g;
   $res =~ s/&oacute;/o/g;
   $res =~ s//a/g;
   $res =~ s/à/a/g;
   $res =~ s/Miko(.*)j Kopernik/Mikolaj Kopernik/g; 
   $res =~ s/&#324;/ne/g;
   $res =~ s/&aelig;/ae/g;
   $res =~ s/UnitedStates/United States/g;
   return $res;
}


# Recursive procedure that extracts information about a given ID an 
# Recursivelly process all the advisor(s) (if any)
sub process_id{
   $id=$_[0];
   
   if ($id_hash{$id}) {
      return;
   }

   $id_hash{$id}=1;
   
   # Emulate a browser for connecting to the site
   $browser = LWP::UserAgent->new(ssl_opts => { verify_hostname => 0 });
   $browser->agent('EJ/0.1');
   # Type d'encodage caractres accept
   $browser->default_header( 'Accept-Charset', 'iso-8859-15, utf-8;q=0.5, *;q=0.5');
   # Type de langage accept
   $browser->default_header( 'Accept-Language', 'fr, en');

   $url="https://genealogy.math.ndsu.nodak.edu/id.php?id=$id";

   $response = $browser->get($url);

   #get raw HTML answer (UTF 8)
   $res=$response->decoded_content;
   #get raw HTML answer (non UTF8)
   #$res=$response->content;

   
   $res=~ s/\n//g;
   

   #Parse HTML page and extract relevant informations
   ($name)=($res =~m|<h2 style="text-align: center; margin-bottom: 0.5ex; margin-top: 1ex">\s*(\S.*\S)\s*</h2>|); $res=$';
    ($univ,$year)=($res=~ m|006633; margin-left: 0.5em">([^<]*)</span>([^<]*)</span>|);
   ($country)=($res=~ m|<img src="img/flags/\w+\.gif" alt="([^"]*)" width="\d+" height="\d+" style="border: 0; vertical-align: middle" title|); 
   ($title)=($res=~m|thesisTitle">([^<]*)</span></div>|);    
   $univ="" if(!$univ);
   $year="" if(!$year);
   $country="" if(!$country);
   $title="" if(!$title);

   $string=html2txt("$id:\n\t$name\n\t$univ\n\t$year\n\t$country\n\t$title\n");
   print "$string";
   

   #print "$remaining";
   #intialize adviosor list
   my @advisor_list=();
   $remaining=$';
   # find advisors of current ID
   while (($adv_id,$adv_name)=($remaining=~ m#Advisor[^:]*: <a href="id.php\?id=(\d+)">([^<]*)</a>#)){
      $remaining=$';
      $string=html2txt("\t<-$adv_id: $adv_name\n");
      print "$string";
      push(@advisor_list,$adv_id);
      #print $remaining;
   }
   
   # recursively process advisors
   foreach $adv_id (@advisor_list) {
      process_id($adv_id);
   }

}

#Main 
#Chacj command line
die("$0 <Person ID on http://genealogy.math.ndsu.nodak.edu>\nExample for pocessing Hibert genealogy: $0 7898") if($#ARGV!=0);

#get id from command line and process it
$id=$ARGV[0];
process_id($id);




